/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.Event;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.collections.ArrayCollection;
  import mx.controls.Menu;
  import mx.events.FlexEvent;
  import mx.events.MenuEvent;
  import mx.states.AddChild;
  import mx.states.SetEventHandler;
  import mx.states.State;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.utils.GraphicUtils;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
  import org.ow2.orchestra.designer.events.SequenceFlowEvent;

  public class ProcessEditorCreatingFlowStateHelper extends ProcessEditorFlowStateHelper {

    public function ProcessEditorCreatingFlowStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }

    /**
     * Generate "creating flow" state.
     * In the creating flow state:
     * - the flow is drawn, from source element to mouse cursor
     * - on mouse move (while the button is down), the flow is updated : the head of the flow follows the mouse cursor.
     * - on clic, if the clic is on a compatible element, the flow is drawn between source and target elements . This view goes back to the "default" state.
     */
    public function addCreatingFlowState():void {
      var state:State = new State();
      state.name = "creatingFlow";
      processEditorView.states.push(state);

      var addArrowChild:AddChild = new AddChild(processEditorView, processEditorView.arrow);
      state.overrides.push(addArrowChild);

      // handler for mouse move event
      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = creatingFlowMouseMove;
      state.overrides.push(mouseMoveListener);

      // handler for mouse click event
      var mouseClickListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.CLICK);
      mouseClickListener.handlerFunction = creatingFlowMouseClick;
      state.overrides.push(mouseClickListener);

      // handler for state enter: resets the arrow position
      state.addEventListener(FlexEvent.ENTER_STATE, creatingFlowStateEnter);
      state.addEventListener(FlexEvent.EXIT_STATE, creatingFlowStateExit);
    }

    /**
     * handler for mouse move events when this view is in the "creating flow" state.
     */
    private function creatingFlowStateEnter(event:FlexEvent):void {
      processEditorView.hideElementContextPanel();

      // Get mouse position.
      var mousePoint:Point = new Point(processEditorView.mouseX, processEditorView.mouseY);

      creatingFlowMoveArrow(mousePoint);
    }

    private function creatingFlowStateExit(event:FlexEvent):void {
      resetCursor();
    }

    /**
     * handler for mouse move events when this view is in the "creating flow" state.
     */
    private function creatingFlowMouseMove(event:MouseEvent):void {
      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      creatingFlowMoveArrow(mousePoint);

      this.updateDropMouseCursor(this.allowFlowTarget(event));
    }

    private function creatingFlowMoveArrow(mousePoint:Point):void {
      // Get coordinates of the center of the source element
      var length:int = processEditorView.processEditorModel.selectedElements.length;
      var element:AbstractElementWithPosition = processEditorView.processEditorModel.selectedElements.getItemAt(0) as
                                                AbstractElementWithPosition;
      var sourceView:AbstractElementView = processEditorView.processView.getElementViewByModel(element);
      var sourceImage:DisplayObject = sourceView.borderImage;
      var sourcePoint:Point = GraphicUtils.findIntersection(
              sourceImage,
              processEditorView.processView.globalToLocal(sourceView.localToGlobal(new Point(sourceImage.x + sourceImage.width
                      / 2, sourceImage.y + sourceImage.height / 2))),
              mousePoint,
              processEditorView.processView);

      processEditorView.arrow.setSourceAndTarget(sourcePoint, mousePoint);
    }

    /**
     * handler for mouse click events when this view is in the "creating flow" state.
     * Select the element under the mouse cursor and create a flow to this element.
     * The view goes back to the "default" state.
     */
    private function creatingFlowMouseClick(event:MouseEvent):void {
      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      // check if mouse is on a element view or on the canvas.
      trace("creatingFlowMouseClick");

      if (allowFlowTarget(event)) {
        // find list of elements that are contained under the mouse cursor
        var listSelectedElements:ArrayCollection = getElementUnderMouse(event);
        if (listSelectedElements.length == 1) {
          //Dispatch creating flow event in order to create flow from source element to target element
          var targetElement:AbstractElementWithPosition =
                  listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
          // get the source element of flow : this is the only selected element
          var elementSource:AbstractElementWithPosition =
                  processEditorView.processEditorModel.selectedElements.getItemAt(0) as AbstractElementWithPosition;


          if (targetElement is PoolModel) {
            var operations:ArrayCollection =
                    ((targetElement as PoolModel).interfaces.values.getItemAt(0) as InterfaceModel).operations.values;
            if (operations.length > 1) {
              var menu:Menu = Menu.createMenu(processEditorView, operations);
              menu.labelField = "name";
              menu.automationName = "PoolOperationChooserMenu";
              menu.addEventListener(MenuEvent.ITEM_CLICK, function(menuEvent:Event):void {
                new ElementPropertyChangeEvent(
                        ElementPropertyChangeEvent.PROPERTY_CHANGED,
                        elementSource,
                        "operation",
                        (menuEvent as MenuEvent).item
                        ).dispatch();
              });
              menu.show(processEditorView.localToGlobal(mousePoint).x, processEditorView.localToGlobal(mousePoint).y);
            }
            if (operations.length > 0) {
              new ElementPropertyChangeEvent(
                      ElementPropertyChangeEvent.PROPERTY_CHANGED,
                      elementSource,
                      "operation",
                      operations.getItemAt(0)
                      ).dispatch();
            }
          } else {
            new SequenceFlowEvent(
                    SequenceFlowEvent.FLOW_CREATION,
                    elementSource,
                    targetElement,
                    processEditorView.processEditorModel
                    ).dispatch();
          }
        }
      }
      // go to default state
      processEditorView.currentState = "default";
    }
  }
}
